#
# File:		AceDB.py
#
# Copyright (C)	Thomas Sean Powell & Siegfried Schloissnig
#
#  -------------------------------------------------------------------
#  Ace-python is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#  or see the on-line version at http://www.gnu.org/copyleft/gpl.txt
#  -------------------------------------------------------------------
#
# Author:	Siegfried Schloissnig
#
# Description:	Interface to the AceDB socket server	
#
# Changes: 	...
#
# Todo: 	add better error feedback		
#

### import some stuff
import sys
import socket
import struct
import md5
import string

import AceMessagePacket

### defaults for connect

ACESERVER_HOST		= '127.0.0.1'
ACESERVER_PORT 		= 23100
ACESERVER_USER		= 'anonymous'
ACESERVER_PASSWORD	= 'guest'

ACESERVER_HELLO		= 'bonjour'

### convert an md5 encrypted string in
### its hex representation
def hexstr(s):
        h = string.hexdigits
        r = ''
        for c in s:
                i = ord(c)
                r = r + h[(i >> 4) & 0xF] + h[i & 0xF]
        return r

def connect(host = ACESERVER_HOST,
	    port = ACESERVER_PORT,
	    user = ACESERVER_USER,
	    passwd = ACESERVER_PASSWORD):
	db = AceDB(host, port, user, passwd)
	if db.connect():
		return db
	else:
 		return None

class AceDB:
	### constructor
	def __init__(self, host, port, user, passwd):
		self.host = host	# where's the server
		self.port = port	# port number the server listens to
		self.clientID = 0	# our client id
		self.socket = None	# the socket
		self.user = user	# connect as
		self.passwd = passwd	# with password
		self.maxBytes = 256	# default for the maxBytes field

	### read the next packet from the socket	
	def __getPacket(self):
		# read the message header
		thePacket = self.socket.recv(50)
		
		# unpack the header, so we can access the fields
		packetFields = struct.unpack("iiiii", thePacket[:20])

		# slice of the message type
		msgType = thePacket[20:50]

		# receive the message body and slice of the '\0' char.
		msgBody = self.socket.recv(packetFields[1], 	# msg body length
					   socket.MSG_WAITALL)[:-1]

		# create a new AceMessagePacket and return it
		return AceMessagePacket.create(msgType, 
						msgBody, 
						packetFields[3],  # clientID 
						packetFields[4])  # max Bytes

	### connect to the server
	def connect(self):
		try:
			# return value
			retval = 1

			# create a new socket object
			self.socket = socket.socket(socket.AF_INET, 
						    socket.SOCK_STREAM)

			# and try to connect to the server
			self.socket.connect(self.host, self.port)

			# try to initiate sign on
			self.socket.send(
				AceMessagePacket.create(
					AceMessagePacket.ACESERV_MSGREQ, 
					ACESERVER_HELLO).getData())
			msg = self.__getPacket()

			if msg.isMsgOK():
				# get our client id 
				self.clientID = msg.getClientID()
				self.maxBytes = msg.getMaxBytes()

				# calculate the md5 user/pasword hash
				hash = md5.new(self.user + self.passwd).digest()
				hash = md5.new(hexstr(hash))
				hash.update(msg.getBody())

				# send the user name and the md5 hash
				self.socket.send(AceMessagePacket.create(
					AceMessagePacket.ACESERV_MSGREQ,
					self.user + ' ' + hexstr(hash.digest()),
					self.clientID,
					self.maxBytes).getData())
			
				msg = self.__getPacket()

				# the server says NO
				if not msg.isMsgOK():
					retval = 0	
			else:
				# initial connect failed
				retval = 0

			return retval 
		except:
			# error occured while connecting
			self.socket = None
			return 0

	### are we connected?
	def isConnected(self):
		return self.socket != None

	# send the server a request and get the result  
	def execute(self, command):
		# send the request to the server
		self.socket.send(AceMessagePacket.create(
				AceMessagePacket.ACESERV_MSGREQ,
				command,
				self.clientID,
				self.maxBytes).getData())

		# get the answer
		msg = self.__getPacket()		
		retval = msg.getBody()

		if msg.isMsgOK():
			# 'normal' message, return the result
			return retval 
		elif msg.isMsgEncore():
			# the answer is to large for one packet
		
			# get the message scraps and put them together	
			while msg.isMsgEncore():
				self.socket.send(aceMessagePacket.create(
					AceMessagePacket.ACESERV_MSGENCORE,
					'encore',
					self.clientID,
					self.maxBytes).getData())
				msg = self.__getPacket() 
				retval = retval + msg.getBody()
		elif msg.isMsgKill():
			self.__killed()
			retval = None

		return retval

	### the server disconnected us
	def __killed(self):
		self.socket.close()
		self.socket = None
		self.clientID = None
		self.maxBytes = None

	### our work is done here
	### close the connection
	def disconnect(self):
		self.socket.send(AceMessagePacket.create(
			AceMessagePacket.ACESERV_MSGREQ,
			'quit',
			self.clientID,
			self.maxBytes).getData())
		msg = self.__getPacket() 
		self.socket.close()
		self.__killed()
